<?php

/**
 * @file
 * Provides the Admin UI to help manage the connection between
 * Drupal and Alfresco.
 */

/**
 * Callback to display the Canopy Admin Dashboard.
 */
function canopy__admin__dashboard() {
  drupal_add_css(drupal_get_path('module', 'canopy') . '/css/stylesheets/canopy-admin.css');
  $sections = canopy__admin__sections_available();
  $content = theme('canopy_admin_content_dashboard', $sections);

  return theme('canopy_admin', $content);
}

/**
 * Callback to display the Canopy Portal Configuration.
 */
function canopy__admin__portal() {
  drupal_add_js(drupal_get_path('module', 'canopy') . '/js/canopy.js');
  drupal_add_css(drupal_get_path('module', 'canopy') . '/css/stylesheets/canopy-admin.css');
  $sections = canopy__admin__sections_available();

  // Get all existing Alfresco sites.
  $alfresco_settings = variable_get('canopy_settings_alfresco', array());

  $alfresco_header = array(
    t("Host"),
    t("Port"),
    t("Service Endpoint"),
    t("Status"),
  );

  if (!empty($alfresco_settings)) {
    $alfresco_rows[] = array(
      'data' => array(
        array(
          'data' => l($alfresco_settings['address'], 'admin/canopy/portal/alfresco/edit'),
        ),
        array(
          'data' => $alfresco_settings['port'],
        ),
        array(
          'data' => $alfresco_settings['endpoint'],
        ),
        array(
          'data' => '<span class="status"><font color="red">Offline</font></span>',
        ),
      ),
    );
  }
  else {
    $alfresco_rows[] = array(
      'data' => array(
        'data' => array(
          'data' => t("You do not have any Alfresco server added."),
          'colspan' => 3,
        ),
      ),
    );
  }

  $alfresco_table = (object) array(
    'content' => theme('table', $alfresco_header, $alfresco_rows),
    'buttons' => array(
      'add' => empty($alfresco_settings) ? TRUE : FALSE,
    ),
  );

  // Get all Drupal settings.
  $drupal_header = array(t("Site Name"), t("Description"), t("Version"));
  $drupal_settings = variable_get('canopy_settings_drupal', array());

  if (!empty($drupal_settings)) {
    $drupal_rows[] = array(
      'data' => array(
        l($drupal_settings['name'], 'admin/canopy/portal/drupal/edit'),
        $drupal_settings['description'],
        VERSION,
      ),
    );
  }
  else {
    $drupal_rows[] = array(
      'data' => array(
        'data' => array(
          'data' => t("You do not have any Drupal site added."),
          'colspan' => 3,
        ),
      ),
    );
  }

  $drupal_table = (object) array(
    'content' => theme('table', $drupal_header, $drupal_rows),
    'buttons' => array('add' => empty($drupal_settings) && !empty($alfresco_settings) ? TRUE : FALSE),
  );

  $table = theme('canopy_admin_content_portal', $alfresco_table, $drupal_table);
  $sidebar = theme('canopy_admin_sidebar', $sections);
  $content = theme('canopy_admin_content', $sidebar, $table);

  return theme('canopy_admin', $content);
}

/**
 * Callback to display the Canopy Portal Configuration Add/Edit form for Alfresco.
 */
function canopy__admin__portal_afresco($server = NULL) {
  drupal_add_css(drupal_get_path('module', 'canopy') . '/css/stylesheets/canopy-admin.css');

  $sections = canopy__admin__sections_available();
  $form = drupal_get_form('canopy__admin__portal_alfresco_modify');
  $sidebar = theme('canopy_admin_sidebar', $sections);
  $content = theme('canopy_admin_content', $sidebar, $form);

  return theme('canopy_admin', $content);
}

/**
 * Callback to display the Canopy Portal Configuration Add/Edit form for Drupal.
 */
function canopy__admin__portal_drupal($site = NULL) {
  drupal_add_css(drupal_get_path('module', 'canopy') . '/css/stylesheets/canopy-admin.css');

  $sections = canopy__admin__sections_available();
  $form = drupal_get_form('canopy__admin__portal_drupal_modify');
  $sidebar = theme('canopy_admin_sidebar', $sections);
  $content = theme('canopy_admin_content', $sidebar, $form);

  return theme('canopy_admin', $content);
}

/**
 * Drupal form to handle Add or Edit Alfresco server settings.
 */
function canopy__admin__portal_alfresco_modify() {
  $settings = variable_get('canopy_settings_alfresco', array());

  $form['#prefix'] = '<div id="portal-alfresco"><h3>' . t("Alfresco") . '</h3>';
  $form['#suffix'] = '</div>';

  // Server Address.
  $form['canopy_settings_alfresco'] = array(
    '#tree' => TRUE,
  );

  // Server Port.
  $form['canopy_settings_alfresco']['address'] = array(
    '#type' => 'textfield',
    '#title' => t("Server address"),
    '#description' => t("Address of the Alfresco server you are connecting to, e.g. localhost or IP address. Do not include http:// or leading slashes."),
    '#required' => TRUE,
    '#size' => 48,
    '#default_value' => !empty($settings['address']) ? $settings['address'] : '127.0.0.1',
  );

  $form['canopy_settings_alfresco']['port'] = array(
    '#type' => 'textfield',
    '#title' => t("Server port"),
    '#description' => t("Server port of the Alfresco server, if any. Leave blank if none available."),
    '#size' => 48,
    '#default_value' => !empty($settings['port']) ? $settings['port'] : '8080',
  );

  // Server service endpoint.
  $form['canopy_settings_alfresco']['endpoint'] = array(
    '#type' => 'textfield',
    '#title' => t("Server service endpoint"),
    '#description' => t("Service endpoint that Alfresco will service any Canopy requests. E.g. 'alfresco/service'. Do not include leading or trailing slashes."),
    '#size' => 48,
    '#default_value' => !empty($settings['endpoint']) ? $settings['endpoint'] : 'alfresco/service',
    '#required' => TRUE,
  );

  $form['canopy_settings_alfresco']['username'] = array(
    '#type' => 'textfield',
    '#title' => t("API username"),
    '#description' => t("API credential used to acquire a token for request any Alfresco services."),
    '#size' => 48,
    '#default_value' => !empty($settings['username']) ? $settings['username'] : '',
    '#required' => TRUE,
  );

  $form['canopy_settings_alfresco']['password'] = array(
    '#type' => 'textfield',
    '#title' => t("API secret"),
    '#description' => t("API credential used to acquire a token for request any Alfresco services."),
    '#size' => 48,
    '#default_value' => !empty($settings['password']) ? $settings['password'] : '',
    '#required' => TRUE,
  );

  $form['buttons'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div class="buttons">',
    '#suffix' => '</div>',
  );

  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('save'),
    '#attributes' => array('class' => 'button ok'),
    '#weight' => 0,
  );

  $form['buttons']['cancel'] = array(
    '#type' => 'item',
    '#value' => l(t("Cancel"), 'admin/canopy/portal', array('attributes' => array('class' => 'button cancel'))),
    '#weight' => 1,
  );

  $form['buttons']['delete'] = array(
    '#type' => 'button',
    '#value' => t('delete'),
    '#executes_submit_callback' => TRUE,
    '#attributes' => array('class' => 'button delete'),
    '#weight' => 2,
  );

  $form['#redirect'] = 'admin/canopy/portal';

  return $form;
}

/**
 * Submit handler for canopy__admin__portal_alfresco_modify() form.
 */
function canopy__admin__portal_alfresco_modify_submit($form, &$form_state) {
  if ($form_state['values']['op'] == 'save') {
    variable_set('canopy_settings_alfresco', $form_state['values']['canopy_settings_alfresco']);

    // Attempt to login to get a Token to validate a connection establishes.
    $ticket = canopy__alfresco_api__ticket_get(TRUE);

    if (!empty($ticket)) {
      drupal_set_message(t("Connection to Alfresco server established. Server settings saved."), 'status', FALSE);
    }
    else {
      drupal_set_message(t("Unable to contact Alfresco Server. Please contact your site administrator."), 'error', FALSE);
    }
  }
  elseif ($form_state['values']['op'] == 'delete') {
    variable_del('canopy_settings_alfresco');
    $_SESSION['canopy_alfresco_api'] = NULL;

    drupal_set_message(t("Alfresco Server settings deleted."), 'status', FALSE);
  }
}

/**
 * Drupal form to handle Add or Edit Drupal site settings.
 */
function canopy__admin__portal_drupal_modify() {
  $settings = variable_get('canopy_settings_drupal', array());

  $form['#prefix'] = '<div id="portal-drupal"><h3>' . t("Drupal") . '</h3>';
  $form['#suffix'] = '</div>';

  $form['canopy_settings_drupal'] = array(
    '#tree' => TRUE,
  );

  $form['canopy_settings_drupal']['name'] = array(
    '#type' => 'textfield',
    '#title' => t("Site name"),
    '#description' => t("Name of this site that will be registered with Alfresco."),
    '#required' => TRUE,
    '#size' => 48,
    '#default_value' => !empty($settings['name']) ? $settings['name'] : '',
  );

  $form['canopy_settings_drupal']['description'] = array(
    '#type' => 'textarea',
    '#title' => t("Site description"),
    '#description' => t("Additional information about this site."),
    '#default_value' => !empty($settings['description']) ? $settings['description'] : '',
  );

  $form['buttons'] = array(
    '#tree' => TRUE,
    '#prefix' => '<div class="buttons">',
    '#suffix' => '</div>',
  );

  $form['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('save'),
    '#attributes' => array('class' => 'button ok'),
    '#weight' => 0,
  );

  $classes = array('class' => 'button cancel');
  $form['buttons']['cancel'] = array(
    '#type' => 'item',
    '#value' => l(t("Cancel"), 'admin/canopy/portal', array('attributes' => $classes)),
    '#weight' => 1,
  );

  $form['buttons']['delete'] = array(
    '#type' => 'button',
    '#value' => t('delete'),
    '#executes_submit_callback' => TRUE,
    '#attributes' => array('class' => 'button delete'),
    '#weight' => 2,
  );

  $form['#redirect'] = 'admin/canopy/portal';

  return $form;
}

/**
 * Submit handler for canopy__admin__portal_alfresco_modify() form.
 */
function canopy__admin__portal_drupal_modify_submit($form, &$form_state) {
  $values = $form_state['values'];

  if ($values['op'] == 'save') {
    $alfresco_settings = variable_get('canopy_settings_alfresco', array());

    if (empty($alfresco_settings)) {
      drupal_set_message(t("You do not have any Alfresco server added. Drupal cannot register a site with Alfresco if an Alfresco server does not exists."), 'error', FALSE);
      $form_state['redirect'] = 'admin/canopy/portal/drupal/add';
    }
    else {
      // Attempt to register the Drupal site before saving settings.
      $registration = canopy__alfresco_api__site_register($values['canopy_settings_drupal']['name'], $values['canopy_settings_drupal']['description']);

      if ($registration) {
        variable_set('canopy_settings_drupal', $values['canopy_settings_drupal']);
        drupal_set_message(t("Drupal site successfully added to Alfresco."), 'status', FALSE);
      }
      else {
        drupal_set_message(t("Unable to contact Alfresco Server. Please contact your site administrator."), 'error', FALSE);
      }
    }
  }
  elseif ($values['op'] == 'delete') {
    variable_del('canopy_settings_drupal');
    drupal_set_message(t("Drupal settings deleted."), 'status', FALSE);
  }
}

/**
 * Get a list of all available Canopy sections.
 *
 * @return array
 *   An array of all sections with their URL, title, and if they are enabled or not.
 */
function canopy__admin__sections_available() {
  $sections = array();
  $links = array();

  // Check if menu exists.
  $sql = "SELECT l.* FROM {menu_links} l WHERE l.link_path LIKE '%s' AND l.menu_name = '%s'";
  $result = db_query($sql, 'admin/canopy/%%', 'navigation');
  while ($row = db_fetch_object($result)) {
    $links[] = $row->link_path;
  }

  // Account administration.
  $sections[] = (object) array(
    'name' => 'account',
    'title' => t("Account Administration"),
    'path' => 'admin/canopy/account',
    'enabled' => in_array('admin/canopy/account', $links) ? TRUE : FALSE,
  );

  // Portal configuration.
  $sections[] = (object) array(
    'name' => 'portal',
    'title' => t("Portal Configuration"),
    'path' => 'admin/canopy/portal',
    'enabled' => in_array('admin/canopy/portal', $links) ? TRUE : FALSE,
  );

  // Content-Type mapping.
  $sections[] = (object) array(
    'name' => 'mapping',
    'title' => t("Content Type Mapping"),
    'path' => 'admin/canopy/mapping',
    'enabled' => in_array('admin/canopy/mapping', $links) ? TRUE : FALSE,
  );

  // File migration.
  $sections[] = (object) array(
    'name' => 'file',
    'title' => t("File Migration"),
    'path' => 'admin/canopy/file',
    'enabled' => in_array('admin/canopy/file', $links) ? TRUE : FALSE,
  );

  // Report manager.
  $sections[] = (object) array(
    'name' => 'report',
    'title' => t("Report Manager"),
    'path' => 'admin/canopy/report',
    'enabled' => in_array('admin/canopy/report', $links) ? TRUE : FALSE,
  );

  // SSO manager.
  $sections[] = (object) array(
    'name' => 'sso',
    'title' => t("SSO Manager"),
    'path' => 'admin/canopy/sso',
    'enabled' => in_array('admin/canopy/sso', $links) ? TRUE : FALSE,
  );

  // Workflow manager.
  $sections[] = (object) array(
    'name' => 'workflow',
    'title' => t("Workflow Manager"),
    'path' => 'admin/canopy/workflow',
    'enabled' => in_array('admin/canopy/workflow', $links) ? TRUE : FALSE,
  );

  return $sections;
}

/**
 * Generates the proper breadcrumb trail for Canopy.
 *
 * @return string
 *   An properly formatted breadcrumb trail.
 */
function canopy__admin__breadcrumb() {
  $path = arg();
  $breadcrumb = '';

  if ($path[0] == 'admin' && $path[1] == 'canopy') {
    for ($i = count($path); $i > 1; $i--) {
      $trail_path = implode('/', $path);
      $item = menu_get_item($trail_path);

      $trail[$item['path']] = array(
        'title' => $item['path'] == 'admin/canopy' ? t("Dashboard") : $item['title'],
        'path' => $item['path'],
      );

      array_pop($path);
    }

    $trail = array_reverse($trail, TRUE);

    foreach ($trail as $data) {
      if ($_GET['q'] == $data['path']) {
        $breadcrumb .= $data['title'];
      }
      else {
        $breadcrumb .= sprintf("%s | ", l($data['title'], $data['path']));
      }
    }
  }

  return $breadcrumb;
}
